#!/bin/bash

######################
# * Prüft auf zu archivierende Verzeichnisse
# * Bildet Tarball
# * Erstellt MD5 Checksumme für den Tarball
# * Verschiebt Tarball ins Archiv via dsmc archive
#
#       Syntax: docs2archive.sh <Stammverzeichnis der zu archiviernden Ordner> <Job-ID> <dsm.opt> [<Verzeichnis Ready-Dateien>]
#       Bsp.:   docs2archive.sh /san/archiv/goobi_archiv goobi_123456_data_20110808125959 /usr/tivoli/tsm/client/ba/bin/dsm-archiv.opt
#
# benoetigt im hotfolder ein Verzeichnis /docs2archive/, in diesem Verzeichnis muss eine 
# Datei liegen, die einen dem ID-Format entsprechenden Namen traegt
# ID-Format: <Dienst>_<Dienst-ID>_data|meta_<fortlaufende Nummer>
# Bsp.: goobi_123456_data_20110808125959
#
#
######################

SCRIPTNAME=`basename $0`
HOSTNAME=$(hostname)

# Pfad zu dsmc (Archiv Client)
DSMC_BINPATH="/opt/tivoli/tsm/client/ba/bin"

# Logfile
LOGFILE="/var/log/jobs/archive/$(hostname)_archive_processing_`date +%Y%m%d`.log"

# Script zur MD5 Erstellung
CREATE_MD5='perl /opt/scripts/create_md5.pl'

starttime=`date +%s`

# Logfile erstellen
echo "[OK] `date +%Y%m%d\ %T` START PID: $$ $0 $*" >> $LOGFILE 


# Einige checks
if [ $# -lt 3 ]; then
#       echo "[FE] `date +%Y%m%d\ %T` docs2archive.sh ENDE Zu wenig Parameter uebergeben." >> $LOGFILE
		echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: not enough arguments" >> $LOGFILE
        exit 1
fi

if [ ! -e ${DSMC_BINPATH}/dsmc ]; then
#        echo "[FE] `date +%Y%m%d\ %T` docs2archive.sh ENDE Archiv-Client dsmc in $DSMC_BINPATH nicht gefunden." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Could not find archive-client in '${DSMC_BINPATH}'" >> $LOGFILE
        exit 1
fi

# Parameter uebergeben
basefolder=$1
archive_id=$2
tsm_optfile=$3

readyfiledir="${basefolder}/docs2archive"
if [ -n "$4" ]; then
    readyfiledir="${basefolder}/${4}"
fi

# Herausfinden, zu welchem Dienst das Archiv gehoert (z.B. goobi, fotothek, ...)
service=${archive_id%%_*}       # pattern matching, alles vor dem ersten _ wird zurueck gegeben

if [ ! -e $tsm_optfile ]; then
#   echo "[FE] `date +%Y%m%d\ %T` $tsm_optfile existiert nicht." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Could not find '${tsm_optfile}'" >> $LOGFILE
    exit 1
fi

# Lockfile erstellen und PID eintragen
LOCKFILE="/var/locks/"${service}"_docs2archive.lock"
if [ -e $LOCKFILE ]; then
#   echo "[FE] `date +%Y%m%d\ %T` docs2archive.sh ENDE Skript laeuft scheinbar schon, lockfile '$LOCKFILE' existiert." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Script is already running, '${LOCKFILE}' exists." >> $LOGFILE
    `rm -f $LOCKFILE 2>&1 >> $LOGFILE`
    exit 1
else
    echo $$ > $LOCKFILE
fi

# Readydatei umbenennen, Workdatei erstellen
if [ -e ${readyfiledir}/~${archive_id} ]; then
#   echo "[FE] `date +%Y%m%d\ %T` ENDE Work-Datei ${basefolder}/docs2archive/~${archive_id} existiert bereits." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Workfile '${basefolder}/docs2archive/~${archive_id}' already exists." >> $LOGFILE
    `rm -f $LOCKFILE 2>&1 >> $LOGFILE`
     exit 1
elif [ ! -e $readyfiledir/$archive_id ]; then
#   echo "[FE] `date +%Y%m%d\ %T` ENDE Readydatei ${basefolder}/docs2archive/${archive_id} existiert nicht." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Readyfile '${basefolder}/docs2archive/${archive_id}' does not exist." >> $LOGFILE
    `rm -f $LOCKFILE 2>&1 >> $LOGFILE`
    exit 1
else    
	#"Testzwecke"
    # `cp -f $readyfiledir/$archive_id $basefolder/test/$archive_id`
	`mv $readyfiledir/$archive_id $readyfiledir/~$archive_id`
fi

# Setzen des Archivierungspfades
archivelink="/${service}"
`ln -fs $basefolder $archivelink`
tarballpath="${archivelink}/${archive_id}.tar"

# Erzeugen des Tarballs im Archivierungspfad
if [ -h $archivelink ]; then
	tar_cmd="tar -c -p -f $tarballpath -C $basefolder $archive_id"
	`$tar_cmd 2>&1 >> $LOGFILE`
	if [  $? != 0 ]; then
#       echo "[FE] `date +%Y%m%d\ %T` Tarball $tarballpath konnte nicht erstellt werden." >> $LOGFILE
		echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Could not create '${tarballpath}'." >> $LOGFILE
		`rm -f $LOCKFILE 2>&1 >> $LOGFILE`
		exit 1
	fi
#   echo "[OK] `date +%Y%m%d\ %T` Tarball $tarballpath erfolgreich erstellt." >> $LOGFILE
	echo "[OK] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} Created '${tarballpath}' successfully." >> $LOGFILE
	tarsize=`du -m $tarballpath | awk '{print $1}'`
else
#   echo "[FE] `date +%Y%m%d\ %T` ENDE Tarball Zielpfad '$archivelink' ist nicht vorhanden." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: The targetdirectory '${archivelink}' does not exist." >> $LOGFILE
    `rm -f $LOCKFILE 2>&1 >> $LOGFILE`
    exit 1
fi

# MD5 Pruefsumme des Tarballs bilden
tarmd5sum=`nice -n 19 $CREATE_MD5 $tarballpath`
#tarmd5sum=`md5sum $tarballpath 2>> $LOGFILE`
#echo "[OK] `date +%Y%m%d\ %T` MD5: $tarmd5sum" >> $LOGFILE
echo "[OK] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} File: '$tarballpath' MD5: '${tarmd5sum}'" >> $LOGFILE

#tarmd5sum=${tarmd5sum%%[ ]*}    # pattern matching, alles vor dem ersten Leerzeichen wird zurueck gegeben

# Archivieren
dsmc_cmd="$DSMC_BINPATH/dsmc archive $tarballpath -description=\"MD5 $tarmd5sum\" -optfile=$tsm_optfile"
echo "[OK] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} Archive-Command: '${dsmc_cmd}'" >> $LOGFILE
`$dsmc_cmd | awk '{print "[OK] '"\`date +%Y%m%d\ %T\`"' " $0}' 2>&1 >> $LOGFILE`
if [ $? != 0 ]; then
	#echo "[FE] `date +%Y%m%d\ %T` Archivierung von $tarballpath nicht erfolgreich." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Archiving process of '${tarballpath}' was not successfull." >> $LOGFILE
    `rm -f $LOCKFILE 2>&1 >> $LOGFILE`
    exit 1
fi

# Ueberpruefung des Archivierens
dsmc_cmd="$DSMC_BINPATH/dsmc query archive $tarballpath -description=\"MD5 $tarmd5sum\" -optfile=$tsm_optfile"
$dsmc_cmd
if [ $? != 0 ]; then
#   echo "[FE] `date +%Y%m%d\ %T` $tarballpath befindet sich nicht im Archiv." >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Could not find '${tarballpath}' in archive, command: $dsmc_cmd" >> $LOGFILE
	echo "[EE] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} ABORT: Could not find '${tarballpath}' in archive, command: $dsmc_cmd"| mail -s "Retrieve Job $archive_id beendet" kru@slub-dresden.de	
    `rm -f $LOCKFILE 2>&1 >> $LOGFILE`
    exit 1
fi


# Loeschen des Tarballs
`rm -f $tarballpath 2>&1 >> $LOGFILE`

# Loeschen des symbolischen Links
`unlink $archivelink`

# Loeschen des Verzeichnisses vorbereiten, Verschieben der Workdatei
`mv $readyfiledir/~$archive_id $basefolder/docs2remove/$archive_id`

# Loeschen der Lock-Datei 
`rm -f $LOCKFILE 2>&1 >> $LOGFILE`



# Erfolg des Vorgangs im Log bestaetigen und beenden
duration=$((`date +%s`-$starttime))
duration_string=$(($duration/3600))"h "$(($duration %3600 /60))"min "$(($duration%60))"s"
#echo "[OK] `date +%Y%m%d\ %T` ENDE Archivierung $0 $* Tarballgroesse: "$tarsize" MB Dauer: "$duration" s ("$duration_string")." >> $LOGFILE 
echo "[OK] `date +%Y%m%d\ %T` PID: ${$} ${SCRIPTNAME} END Archiveprocessing $0 $* tarballsize: "$tarsize" MB duration: "$duration" s ("$duration_string")" >> $LOGFILE

exit 0


